package	jp.co.sds_corp.syslog.action;

import java.awt.Component;
import java.util.ResourceBundle;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.audio.AudioPlay;
import jp.co.sds_corp.util.common.BlockQueue;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.syslog.SyslogMsg;
import jp.co.sds_corp.util.window.WindowUtility;

/**
Popupを表示し、音を鳴らすアクションです。
音は、PopupのOKボタンで停止します。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR><TD>File</TD><TD>※</TD><TD>音源ファイル</TD><TD>&nbsp;</TD></TR>
	<TR><TD>Time</TD><TD>&nbsp;</TD><TD>回数</TD><TD>0未満を指定するとPopupを消すまで鳴り続ける。</TD></TR>
	<TR>
		<TD>Filter</TD><TD>&nbsp;</TD>
		<TD>処理対象のsyslogを選択するFilter</TD>
		<TD>指定されない場合は、全てのsyslogが対象</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action class="jp.co.sds_corp.hibiya.action.PopupAction"&gt;
	&lt;File&gt;error.wav&lt;/File&gt;
	&lt;Time&gt;1&lt;/Time&gt;
&lt;/Action&gt;
</PRE>
*/
public class PopupAction extends SoundAction
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	ResourceBundle res;
	transient Popup	popup;
	transient Thread	popThr;
	Component parent;

	public PopupAction() {
		res = ResourceBundle.getBundle( "resource.viewer" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
	}

	/**
	* アクションを実行します。
	*/
	public void action( SyslogEvent ev ) {
		if ( popup == null ) {
			popup = new Popup();
			popThr = new Thread( popup );
			popThr.setName( "PopupAction:Popup" );
			popThr.start();
		}
		SyslogMsg msg = ev.getMessage();
		popup.add( msg );
	}

	// どこからも呼ばれない
	public void stop() {
		try {
			popThr.interrupt();
			popThr.join();
		} catch ( Exception ex ) {
		}
	}

	class Popup implements Runnable
	{
		JOptionPane	pane;
		JDialog	dialog;
		BlockQueue<SyslogMsg>	que = new BlockQueue<SyslogMsg>();
		Popup() {
			pane = new JOptionPane();
			String text;
			text = res.getString( "action.popup.title" );
			dialog = pane.createDialog( parent, text );
		}

		void add( SyslogMsg msg ) {
			que.add( msg );
		}

		public void run() {
			for ( ;; ) {
				if ( Thread.currentThread().interrupted()) {
					break;
				}
				try {
					if ( que.isEmpty()) {
						continue;
					}
				} catch ( InterruptedException iex ) {
					break;
				}
				SyslogMsg msg = (SyslogMsg )que.get();
				show( msg );
				que.clear();
			}
		}
		void show( SyslogMsg msg ) {
			try {
				AudioPlay audio = null;
				if ( getFile() != null ) {
					audio = new AudioPlay( getFile(), time );
					audio.start();
				}
				pane.setMessage( msg.getMessage() );
				if ( msg.getPriority().compareTo( Priority.ERR ) <= 0 ) {
					pane.setMessageType( JOptionPane.ERROR_MESSAGE );
				} else
				if ( msg.getPriority().compareTo( Priority.WARNING ) <= 0 ) {
					pane.setMessageType( JOptionPane.WARNING_MESSAGE );
				} else {
					pane.setMessageType( JOptionPane.INFORMATION_MESSAGE );
				}
				dialog.pack();
				if ( parent != null ) {
					WindowUtility.center( parent, dialog );
				}
				dialog.setVisible( true );
				if ( audio != null ) {
					audio.setPlayFlg( false );
				}
			} catch ( Exception ex ) {
				logger.error( this, "error", ex );
			}
		}
	}

	/**
	* 親コンポーネントを設定します。
	* @param	var		設定するデータ
	*/
	public void setParent( Component var ) {
		this.parent = var ;
	}

	/**
	* 回数(time)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTime( int var ) {
		this.time = var ;
	}

}
