package	jp.co.sds_corp.syslog.viewer; 

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.SyslogService;
import jp.co.sds_corp.util.window.ColumnSelectPanel;
import jp.co.sds_corp.util.window.ValuePanel;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link SyslogService}と{@link SyslogViewer}の設定を行なうダイアログです。
*/
public class SetupDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	JButton BTN_OK = new JButton( "OK" );
	JButton BTN_CANCEL = new JButton( "Cancel" );
	JCheckBox CHK_HOST;
	JCheckBox CHK_FRONT;
	JCheckBox CHK_LIMIT;
	ValuePanel	VAL_MAX;
	JCheckBox CHK_POPUP;
	JButton	BTN_POPUP;
	ActionDialog	popupDlg;
	JTabbedPane tp;
	ColumnSelectPanel column;
	ActionPanel action;
	FilterPanel filter;

	SyslogParam		param;
	SyslogService	service;
	ResourceBundle res;

	public SetupDialog( JDialog parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	public SetupDialog( JFrame parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	public SetupDialog( SyslogParam param ) {
		super();
		this.param = param;
		createControls();
	}

	protected void createControls() {
		setModal( true );
		WindowUtility.setHideOnEscape( this );

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		setResizable( false );
		text = res.getString( "setup.title" );
		setTitle( text );
		setSize( 477, 421 );
		getContentPane().setLayout(
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ) );
		setFont( param.getDefaultFont() );

		WindowUtility.setHideOnEscape( this );

		service = SyslogService.getInstance();

		tp = new JTabbedPane();
		JPanel pnl1 = createSetupPanel();
		text = res.getString( "setup.tab1.text" );
		tp.addTab( text, pnl1 );

		JPanel pnl3 = new JPanel();
		pnl3.setLayout( new BorderLayout() );
		pnl3.add( createColumnPanel(), BorderLayout.NORTH );
		text = res.getString( "setup.tab2.text" );
		tp.addTab( text, pnl3 );

		action = new ActionPanel( this, param );
		text = res.getString( "setup.tab3.text" );
		tp.addTab( text, action );
		getContentPane().add( tp );

		Box box1 = new Box( BoxLayout.X_AXIS );
		box1.add( BTN_OK );
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onCancel();
			}
		});
		box1.add( BTN_CANCEL );
		box1.setPreferredSize( new Dimension( 450, 50 ));
		getContentPane().add( box1 );
		getRootPane().setDefaultButton( BTN_OK );
	}

	protected JPanel createSetupPanel() {
		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ) );
		Box box1 = new Box( BoxLayout.X_AXIS );
		Box box2 = new Box( BoxLayout.Y_AXIS );

		String text;
		text = res.getString( "setup.host.text" );
		CHK_HOST = new JCheckBox( text );
		text = res.getString( "setup.front.text" );
		CHK_FRONT = new JCheckBox( text );
		CHK_HOST.setMnemonic( KeyEvent.VK_H );
		box2.add( CHK_HOST );
		CHK_FRONT.setEnabled( false );
		CHK_FRONT.setMnemonic( KeyEvent.VK_F );
		box2.add( CHK_FRONT );
		box1.add( box2 );
		box1.add( Box.createHorizontalGlue() );
		pnl.add( box1 );

		JPanel limit = createLimitPanel();
		text = res.getString( "setup.limit.text" );
		limit.setBorder( new TitledBorder( text ));
		pnl.add( limit );

		popupDlg = new ActionDialog( this, param );
		JPanel popup = createPopupPanel();
		text = res.getString( "setup.popup.text" );
		popup.setBorder( new TitledBorder( text ));
		pnl.add( popup );

		filter = new FilterPanel( this, param );
		text = res.getString( "setup.filter.text" );
		filter.setBorder( new TitledBorder( text ));
		pnl.add( filter );
//		pnl.add( Box.createVerticalStrut( 30 ));
		return pnl;
	}

	protected JPanel createLimitPanel() {
		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ) );

		Box box = new Box( BoxLayout.X_AXIS );
		String text;
		text = res.getString( "setup.limit.use.text" );
		CHK_LIMIT = new JCheckBox( text );
		CHK_LIMIT.setMnemonic( KeyEvent.VK_L );
		CHK_LIMIT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if ( CHK_LIMIT.isSelected()) {
					VAL_MAX.setEnabled( true );
				} else {
					VAL_MAX.setEnabled( false );
				}
			}
		} );
		box.add( CHK_LIMIT );
		box.add( Box.createHorizontalGlue());
		pnl.add( box );
		text = res.getString( "setup.limit.max.text" );
		VAL_MAX = new ValuePanel( ValuePanel.ReturnType.INT, text );
		pnl.add( VAL_MAX );
		return pnl;
	}

	protected JPanel createPopupPanel() {
		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.X_AXIS ) );

		String text;
		text = res.getString( "setup.popup.use.text" );
		CHK_POPUP = new JCheckBox( text );
		CHK_POPUP.setMnemonic( KeyEvent.VK_P );
		CHK_POPUP.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if ( CHK_POPUP.isSelected()) {
					BTN_POPUP.setEnabled( true );
				} else {
					BTN_POPUP.setEnabled( false );
				}
			}
		} );
		pnl.add( CHK_POPUP );
		text = res.getString( "setup.popup.button" );
		BTN_POPUP = new JButton( text );
		BTN_POPUP.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				WindowUtility.center( SetupDialog.this, popupDlg );
				popupDlg.setVisible( true );
			}
		} );
		pnl.add( BTN_POPUP );
		pnl.add( Box.createHorizontalGlue() );
		return pnl;
	}

	protected JPanel createColumnPanel() {
		column = new ColumnSelectPanel();
		return column;
	}

	public void setVisible( boolean var ) {
		if ( !var ) {
			super.setVisible( false );
			return;
		}
		column.setChange( false );
		filter.setChange( false );
		column.setColumn( param.getAllColumn(), param.getSelectedColumn() );
		filter.clearFilter();
		filter.addFilterList( param.listFilters() );
		action.clearAction();
		action.addActionList( service.getActionList() );
		CHK_HOST.setSelected( param.getHostAsName() );
		CHK_LIMIT.setSelected( param.getLimitLogSize() );
		VAL_MAX.setValue( String.valueOf( param.getMaxLogSize() ));
		if ( !param.getLimitLogSize() ) {
			VAL_MAX.setEnabled( false );
		}
		CHK_POPUP.setSelected( param.getPopup().isValid() );
		BTN_POPUP.setEnabled( param.getPopup().isValid() );
		popupDlg.setAction( param.getPopup() );
		tp.setSelectedIndex( 0 );
		BTN_OK.requestFocus();
		super.setVisible( true );
	}

	public boolean isFilterChange() {
		return filter.isChange();
	}

	public boolean isColumnChange() {
		return column.isChange();
	}

	public void onOk() {
		if ( column.isChange() ) {
			param.setColumn( column.getSelected() );
		}
		param.setHostAsName( CHK_HOST.isSelected() );
		param.setLimitLogSize( CHK_LIMIT.isSelected() );
		Integer ival = (Integer )VAL_MAX.getValue();
		if ( ival != null ) {
			param.setMaxLogSize( ival.intValue() );
		}
		param.getPopup().setValid( CHK_POPUP.isSelected() );

		if ( filter.isChange() ) {
			param.listFilters().clear();
			param.listFilters().addAll( filter.listFilters() );
		}
		if ( action.isChange() ) {
			service.getActionList().clear();
			service.getActionList().addAll( action.listActions() );
		}
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}
}
