package jp.co.sds_corp.syslog.viewer;

import java.text.SimpleDateFormat;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.syslog.filter.FilterUtility;
import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* {@link SyslogTable}のデータモデルです。
*/
public class SyslogModel extends AbstractTableModel
{
	private static final long serialVersionUID = 1L;
	Vector<SyslogEvent>	vlog = new Vector<SyslogEvent>();
	SyslogParam param;
	boolean bOrd = true;

	SyslogModel( SyslogParam param ) {
		this.param = param;
	}

	protected void clear() {
		vlog.clear();
	}

	protected void add( int pos, Object obj ) {
		SyslogEvent ev = (SyslogEvent )obj;
		java.util.List<IFilter> l = param.listFilters();
		boolean bFilter = FilterUtility.isFiltered( l, ev );
		if ( !bFilter ) {
			vlog.add( pos, (SyslogEvent )obj );
		}
	}

	protected void remove( Object obj ) {
		vlog.remove( obj );
	}

	protected Object getData( int nPos ) {
		if ( bOrd != true ) {
			nPos = vlog.size() - nPos - 1;
		}
		return vlog.get( nPos );
	}

	public int getColumnCount() {
		return param.getColumnCount();
	}
	public String getColumnName( int nClm ) {
		return param.getColumnName( nClm );
	}
	public int getRowCount() {
		return vlog.size();
	}

	public Object getValueAt( int row, int col ) {
		int		nRow;
		if ( bOrd ) {
			nRow = row;
		} else {
			nRow = vlog.size() - row - 1;
		}
		if ( vlog.size() <= nRow ) {
			return null;
		}
		SyslogEvent ev = (SyslogEvent )vlog.get( nRow );
		SyslogMsg msg = ev.getMessage();
		String name = param.getColumnName( col );
/*
		if ( name.equals( " " )) {
			return Integer.valueOf( nRow );
		} else
*/
		if ( name.equals( "Date" )) {
			SimpleDateFormat sdf1 = new SimpleDateFormat( "yyyy/MM/dd" );
			return sdf1.format( msg.getDate().getTime() );
		} else if ( name.equals( "Time" )) {
			SimpleDateFormat sdf2 = new SimpleDateFormat( "HH:mm:ss.SSS" );
			return sdf2.format( msg.getDate().getTime() );
		} else if ( name.equals( "Host" )) {
			if ( param.getHostAsName() ) {
				return msg.getAddress().getHostName();
			} else {
				return msg.getAddress().getHostAddress();
			}
		} else if ( name.equals( "Priority" )) {
			return msg.getPriority().toString();
		} else if ( name.equals( "Facility" )) {
			return msg.getFacility().toString();
		} else if ( name.equals( "OrgTime" )) {
			return msg.getOrgTime();
		} else if ( name.equals( "Tag" )) {
			return msg.getTag();
		} else if ( name.equals( "Ident" )) {
			return msg.getIdent();
		} else if ( name.equals( "Message" )) {
			return msg.getMessage();
		}
		return null;
	}
	public boolean isCellEditable( int row, int col ) {
		return false;
	}
	public void reverse() {
		setOrder( !bOrd );
	}
	public void setOrder( boolean ord ) {
		bOrd = ord;
	}
	public boolean getOrder() {
		return bOrd;
	}
}

