package	jp.co.sds_corp.syslog.viewer.filter;

import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.filter.PatternFilter;
import jp.co.sds_corp.util.window.ValuePanel;

/**
* {@link PatternFilter}の設定を行います。
*/
public class PatternPanel extends AbstractPanel
{
	private static final long serialVersionUID = 1L;
	ValuePanel	VAL_PTN;
	JCheckBox	CHK_CASE;
	JCheckBox	CHK_EXC;
	JCheckBox	CHK_VALID;
	ResourceBundle res;

	public PatternPanel() {
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "filter.pattern.title" );
		setBorder( new TitledBorder( text ));
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));

		createControls();
	}

	protected void createControls() {
		String text;
		text = res.getString( "filter.exclude.text" );
		CHK_EXC = new JCheckBox( text );
		text = res.getString( "filter.valid.text" );
		CHK_VALID = new JCheckBox( text );
		text = res.getString( "filter.pattern.lbl2.text" );
		CHK_CASE = new JCheckBox( text );

		text = res.getString( "filter.pattern.lbl1.text" );
		VAL_PTN = new ValuePanel( ValuePanel.ReturnType.STRING, text );
		add( VAL_PTN );
		Box box2 = new Box( BoxLayout.X_AXIS );
		box2.add( CHK_CASE );
		box2.add( Box.createHorizontalStrut( 500 ));
		add( box2 );
		Box box3 = new Box( BoxLayout.X_AXIS );
		box3.add( CHK_EXC );
		box3.add( Box.createHorizontalStrut( 500 ));
		add( box3 );
		Box box4 = new Box( BoxLayout.X_AXIS );
		box4.add( CHK_VALID );
		box4.add( Box.createHorizontalStrut( 500 ));
		add( box4 );
	}

	public void clear() {
		this.filter = null;
		VAL_PTN.setValue( null );
		CHK_CASE.setSelected( false );
		CHK_EXC.setSelected( false );
		CHK_VALID.setSelected( false );
	}

	public void setFilter( IFilter filter ) {
		super.setFilter( filter );
		if ( filter == null ) {
			clear();
			return;
		}
		PatternFilter sf = (PatternFilter )filter;
		VAL_PTN.setValue( sf.getPattern() );
		if ( sf.isIgnoreCase()) {
			CHK_CASE.setSelected( true );
		} else {
			CHK_CASE.setSelected( false );
		}
		if ( sf.isExclude()) {
			CHK_EXC.setSelected( true );
		} else {
			CHK_EXC.setSelected( false );
		}
		if ( sf.isValid()) {
			CHK_VALID.setSelected( false );
		} else {
			CHK_VALID.setSelected( true );
		}
	}

	public void updateFilter() {
		PatternFilter sf;
		if ( filter == null ) {
			sf = new PatternFilter();
			filter = sf;
		} else {
			sf = (PatternFilter )filter;
		}
		sf.setPattern( (String )VAL_PTN.getValue() );
		sf.setIgnoreCase( CHK_CASE.isSelected());
		sf.setExclude( CHK_EXC.isSelected());
		sf.setValid( !CHK_VALID.isSelected());
	}

}
