package	jp.co.sds_corp.syslog.action;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
受信したsyslogを、Log4jを使用して保存するアクションです。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR>
		<TD>Config</TD><TD>&nbsp;</TD>
		<TD>Log4jの設定ファイル</TD>
		<TD>ここで指定すると、全体の設定も変更される</TD>
	</TR>
	<TR>
		<TD>Layout</TD><TD>&nbsp;</TD>
		<TD>formatに使用するLayout</TD>
		<TD>指定されない場合はメッセージのみ出力</TD>
	</TR>
	<TR>
		<TD>Filter</TD><TD>&nbsp;</TD>
		<TD>処理対象のsyslogを選択するFilter</TD>
		<TD>指定されない場合は、全てのsyslogが対象</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action class="jp.co.sds_corp.syslog.action.Log4jAction"&gt;
	&lt;Config&gt;config/log4j.xml&lt;/Config&gt;
	&lt;Layout class="jp.co.sds_corp.syslog.layout.SyslogLayout"&gt;
		%d %h %f:%p [%m]
	&lt;/Layout&gt;
&lt;/Action&gt;
</PRE>
*/
public class Log4jAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	String	config;

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
/*
		setConfig( config.get( "Config", null ));
		if ( getConfig() != null ) {
			CmnLog.configure( getConfig() );
		}
*/
	}

	/**
	* アクションを実行します。
	*/
	public void action( SyslogEvent ev ) {
		SyslogMsg msg = ev.getMessage();
		String s;
		if ( getLayout() == null ) {
			s = msg.getMessage();
		} else {
			s = getLayout().format( msg );
		}
		if ( msg.getPriority().compareTo( Priority.ERR ) <= 0 ) {
			logger.error( this, s );
		} else
		if ( msg.getPriority().compareTo( Priority.WARNING ) <= 0 ) {
			logger.warn( this, s );
		} else
		if ( msg.getPriority().compareTo( Priority.INFO ) <= 0 ) {
			logger.info( this, s );
		} else {
			logger.debug( this, s );
		}
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = super.paramString();
		if ( config != null ) {
			ret += config;
		}
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		if ( getConfig() != null ) {
			c = config.addChild( "Config", getConfig() );
		}
	}

	/**
	* 設定ファイル名(config)を設定します。
	* @param	var		設定するデータ
	*/
	public void setConfig( String var ) {
		this.config = var ;
	}
	/**
	* 設定ファイル名(config)を取得します。
	* @return	fileの内容
	*/
	public String getConfig() {
		return config;
	}

}
